/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.5
  * @date     2021-12-17
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "tmr.h"
#include "usart.h"
#include "flash.h"
#include "ota.h"

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  /* config vector table offset */ 
  nvic_vector_table_set(NVIC_VECTTAB_FLASH, 0x4000);
 
  /* config nvic priority group */
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
 
  system_clock_config();
  at32_board_init();

  /* check and set ota upgrade success flag */
  ota_init();

  /* init usart used for app update */
  uart_init(115200);
  
  /* init tmr used for show code running state(led cycle toggle) */
  tmr_init();

  while(1)
  {
    ota_upgrade_app_handle();
  }
}
