/**
  **************************************************************************
  * @file     ota.h
  * @version  v2.0.5
  * @date     2021-12-17
  * @brief    ota header file
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#ifndef __OTA_H__
#define __OTA_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "at32f403a_407_board.h"

/* app starting address */
#define APP1_START_ADDR                  0x08004000

/* the previous sector of app starting address is ota upgrade flag */
#define OTA_UPGRADE_APP1_FLAG_ADDR       (APP1_START_ADDR - 0x800)
#define OTA_UPGRADE_APP2_FLAG_ADDR       (APP1_START_ADDR - 0x1000)

/* when app received cmd 0x5aa5 from pc-tool, will set up the flag,
indicates that an app upgrade will follow, see ota application note for more details */
#define OTA_UPGRADE_APP2_FLAG            0xABCD0002

/**
  * @brief  cmd data group type
  */
typedef struct
{
  uint8_t cmd_head;
  uint8_t cmd_addr[4];
  uint8_t cmd_buf[0x800];
  uint8_t cmd_check;
} cmd_data_group_type;

/**
  * @brief  cmd data step type
  */
typedef enum
{
  CMD_DATA_IDLE,
  CMD_DATA_ADDR,
  CMD_DATA_BUF,
  CMD_DATA_CHACK,
  CMD_DATA_DONE,
  CMD_DATA_ERR,
} cmd_data_step_type;

/**
  * @brief  cmd ctr step type
  */
typedef enum
{
  CMD_CTR_IDLE,
  CMD_CTR_INIT,
  CMD_CTR_DONE,
  CMD_CTR_ERR,
  CMD_CTR_APP,
} cmd_ctr_step_type;

/**
  * @brief  update status type
  */
typedef enum
{
  UPDATE_PRE,
  UPDATE_CLEAR_FLAG,
  UPDATE_ING,
  UPDATE_DONE,
} update_status_type;

typedef void (*otafun)(void);

extern cmd_data_group_type cmd_data_group_type_struct;
extern cmd_data_step_type cmd_data_step;
extern cmd_ctr_step_type cmd_ctr_step;
extern update_status_type update_status;

void command_handle(void);
void back_err(void);
void back_ok(void);
void ota_upgrade_app_handle(void);
void app_load(uint32_t appxaddr);
void ota_init(void);

#ifdef __cplusplus
}
#endif

#endif
