/**
  **************************************************************************
  * @file     ota.c
  * @version  v2.0.5
  * @date     2021-12-17
  * @brief    ota program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "ota.h"
#include "flash.h"
#include "tmr.h"

otafun jump_to_app;
uint8_t update_result = 0; //1=fail

/* app_load don't optimize */
#if defined (__CC_ARM)
  #pragma O0
#elif defined (__ICCARM__)
  #pragma optimize=s none
#endif
/**
  * @brief  app load.
  * @param  app_addr
  *         app code starting address
  * @retval none
  */
void app_load(uint32_t app_addr)
{
  /* check app starting address whether 0x08xxxxxx */
  if(((*(uint32_t*)(app_addr + 4)) & 0xFF000000) != 0x08000000)
    return;
  /* check the address of stack */
  if(((*(uint32_t*)app_addr) - 0x20000000) < (SRAM_SIZE * 1024))
  {
    crm_reset();
   
    /* disable periph clock */
    crm_periph_clock_enable(CRM_TMR3_PERIPH_CLOCK, FALSE);
    crm_periph_clock_enable(CRM_GPIOA_PERIPH_CLOCK, FALSE);
    
    /* disable nvic irq and clear pending */
    nvic_irq_disable(TMR3_GLOBAL_IRQn);
    __NVIC_ClearPendingIRQ(TMR3_GLOBAL_IRQn);
   
    jump_to_app = (otafun)*(uint32_t*)(app_addr + 4);        /* code second word is reset address */    
    __set_MSP(*(uint32_t*)app_addr);                        /* init app stack pointer(code first word is stack address) */
    jump_to_app();                                          /* jump to user app */
  }
}

/**
  * @brief  ota upgrade app handle.
  * @param  none
  * @retval none
  */
void ota_upgrade_app_handle(void)
{
  uint8_t i;
  if(flash_upgrade_flag_read() != SET)
    return;
  for(i = 0; i < 3; i++)
  {
    if(app_flash_update()==SUCCESS)
    {
      app_load(APP_START_ADDR);
    }
  }
  update_result = 1;
}
